/*

communication library for sending function calls (with memory interfacing) via serial link

messageframe:
bytes	desciption
---------------------------
1		startcharacter
2		length of complete telegram in WORD (hibyte_lobyte), so maximum of 64k can be transferred with one telegram
2		commandcode

depending on the command following parameters are sent after the initial block

1		char, byte
2		WORD, signed int (hibyte_lobyte)
4		LONG, signed long

2 + bs	datablock: word (hibyte_lobyte) length, following data (i.e. strings, memory blocks)


attention: the dataformat is not robust against packetmalforming, so either the format has to be packed inside
"safe" frames, i.e. tcp, or telegram uncertainty is allowed (debugging, ..)

*/

#include "common_definitions.h"

// helper functions
// setting of data inside the stream
WORD SetByte(CHAR *buffer, WORD pos, CHAR v)
{
	buffer[pos++] = v;
	return pos;
}

WORD SetWord(CHAR *buffer, WORD pos, WORD v)
{
	buffer[pos++] = (v >> 8) & 0xFF;
	buffer[pos++] = v & 0xFF;
	return pos;
}

WORD SetLong(CHAR *buffer, WORD pos, LONG v)
{
	buffer[pos++] = (v >> 24) & 0xFF;
	buffer[pos++] = (v >> 16) & 0xFF;
	buffer[pos++] = (v >> 8) & 0xFF;
	buffer[pos++] = v & 0xFF;
	return pos;
}

WORD SetBuffer(CHAR *buffer, WORD pos, CHAR *v, WORD len)
{
	WORD loop;

	if (v == NULL) {
		len = 0;
	} else {
		if (len == -1) {
			len = strlen((char*)v);
		}
	}

	pos = SetWord(buffer, pos, len);

	for (loop = 0; loop < len; loop++) {
		buffer[pos++] = v[loop];
	}
	return pos;
}



// getting the data out of the stream
WORD GetByte(CHAR *buffer, WORD pos, CHAR *v)
{
	*v = buffer[pos++];
	return pos;
}

WORD GetWord(CHAR *buffer, WORD pos, WORD *v)
{
	*v = buffer[pos++];	*v = *v << 8;
	*v = *v + buffer[pos++];
	return pos;
}

WORD GetLong(CHAR *buffer, WORD pos, LONG *v)
{
	LONG v1, v2, v3;
	v1 = buffer[pos++];	v1 = v1 << 24;
	v2 = buffer[pos++];	v2 = v2 << 16;
	v3 = buffer[pos++];	v3 = v3 << 8;
	*v = v1 + v2 + v3 + buffer[pos++];

	return pos;
}

WORD GetBuffer(CHAR *buffer, WORD pos, CHAR *v, WORD *len)
{
	WORD loop;

	pos = GetWord(buffer, pos, len);

	for (loop = 0; loop < *len; loop++) {
		v[loop] = buffer[pos++];
	}

	return pos;
}


WORD GetUsedLength(CHAR *buffer)
{
	WORD w;
	int pos = 1;
	GetWord(buffer, pos, &w);

	return w;
}

void SetUsedLength(CHAR *buffer, WORD w)
{
	int pos = 1;
	SetWord(buffer, pos, w);
}

// creates a message, memory has to be allocated
// return: current length of messagestream
WORD CreateMessage(CHAR *buffer, WORD commandcode)
{
	int len = 0;
	len = SetByte(buffer, len, STX_CHAR);
	len = SetWord(buffer, len, 0x0002); // pre - init the length of the telegram (only commandcode)
	len = SetWord(buffer, len, commandcode);
	return len;
}

void AddByteParm(CHAR *buffer, CHAR v)
{
	WORD pos;
	pos = GetUsedLength(buffer) + 3;
	pos = SetByte(buffer, pos, v);
	SetUsedLength(buffer, pos - 3);
}

void AddWordParm(CHAR *buffer, WORD v)
{
	WORD pos;
	pos = GetUsedLength(buffer) + 3;
	pos = SetWord(buffer, pos, v);
	SetUsedLength(buffer, pos - 3);
}

void AddLongParm(CHAR *buffer, LONG v)
{
	WORD pos;
	pos = GetUsedLength(buffer) + 3;
	pos = SetLong(buffer, pos, v);
	SetUsedLength(buffer, pos - 3);
}

void AddBufferParm(CHAR *buffer, CHAR *v, WORD len)
{
	WORD pos;
	pos = GetUsedLength(buffer) + 3;
	pos = SetBuffer(buffer, pos, v, len);
	SetUsedLength(buffer, pos - 3);
}

