/*


  topfcomm

  send and receive encoder/decoder for the function via seriallink communication
  every TAP function should have a corresponding communication function here

  every TAP function consists of following communication functions:

  CTAP_[functionname]						- command word for tap
  STAP_[functionname](...parameters...)		- send tap command
  ETAP_[functionname](...parameters...)		- execute tap command (topfield only, execution of the command and answering)
  ATAP_[functionname](...parameters...)		- decode answer of tap command


  a single function with return parameters generates following sequence:

  PC: allocating buffer (or use preallocated maxbuffer)
  PC: generate tapcommand (commandword CTAP_) buffer (via STAP_)
  PC: send buffer over serial line
  TP: receives command, allocates memory according to length, or use preallocated buffer
  TP: decodes command word (CTAP_) and call ETAP_. ETAP_ generates the answerbuffer
  TP: send the answerbuffer to the PC
  PC: receives the answercommand (with CTAP_ as commandword) and decodes the parameters of the answer (or no parameters if setonly)


*/

#include "common_definitions.h"
#include "tap_definitions.h"

#define RUN_TOPF
#ifdef _WIN32
	#undef		RUN_TOPF
#else
#include "tap.h"
#include "tap_topfield.h"
#endif


// static buffer definition for input and output buffers
CHAR ib[0x10000];
CHAR ob[0x10000];


//void ETAP_GetState();
//void STAP_GetState();
//void ATAP_GetState(LONG *state, LONG *subState);


WORD ___asctohex[] =
{
  0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99,
  0xAA,0xBB,0xCC,0xDD,0xEE,0xFF,
  0x00,0xAA,0xBB,0xCC,0xDD,0xEE,0xFF
};

/****************************************************************************
 * Converts first to string elements to a unsigned char
 ****************************************************************************/

CHAR ConvAscHex(CHAR c1, CHAR c2)
{
	return (___asctohex[c1-0x30] & 0xf0) +	(___asctohex[c2-0x30] & 0x0f);
}


extern bool _exitFlag;

void SendBuffer()
{
	WORD len, loop;
	WORD pos = GetWord(ob, 1, &len);
	CHAR b[2];
	b[0] = STX_CHAR;
	b[1] = '\0';

	len += 3; //  + stx commandcode

	TAP_Print(b);
	for (loop = 1; loop < len; loop++) {
		TAP_Print("%02X", ob[loop]);
	}
}


// returns -1 if break character is found
int ReceiveBuffer()
{
	WORD len, loop;
	CHAR hi, lo;
	CHAR bb[2];

	// start of text is checked on function entry
	// version2: decode from transmitted ascii characters
	ib[0] = STX_CHAR;
	TAP_PutCh(ib[0]);

	bb[0] = TAP_GetCh(); TAP_PutCh(bb[0]); if (bb[0] == BREAK_CHAR) return -1;
	bb[1] = TAP_GetCh(); TAP_PutCh(bb[1]); if (bb[1] == BREAK_CHAR) return -1;

	hi = ConvAscHex(bb[0], bb[1]); ib[1] = hi;


	bb[0] = TAP_GetCh(); TAP_PutCh(bb[0]); if (bb[0] == BREAK_CHAR) return -1;
	bb[1] = TAP_GetCh(); TAP_PutCh(bb[1]); if (bb[1] == BREAK_CHAR) return -1;

	lo = ConvAscHex(bb[0], bb[1]); ib[2] = lo;

	len = hi;
	len = (len << 8) + lo;

	for (loop = 0; loop < len; loop++) {
		bb[0] = TAP_GetCh(); TAP_PutCh(bb[0]); if (bb[0] == BREAK_CHAR) return -1;
		bb[1] = TAP_GetCh(); TAP_PutCh(bb[1]); if (bb[1] == BREAK_CHAR) return -1;
		ib[loop + 3] = ConvAscHex(bb[0], bb[1]);
	}

	//TAP_Print("<>END<>");

	return 0;
}

void DecodePackage()
{
	WORD cmd;
	GetWord(ib, 3, &cmd);

	switch (cmd) {
		// -- DEBUG FUNCTIONS
		case CTAP_Debug_BufferT2P : ETAP_Debug_BufferT2P(); break;
		case CTAP_Debug_BufferP2T : ETAP_Debug_BufferP2T(); break;
		case CTAP_Debug_QuitDebugger : _exitFlag = TRUE; break;
		
		// -- SYSTEM FUNCTION ---------------------------------------------------
		case CTAP_SystemProc : ETAP_SystemProc(); break;
		case CTAP_GetState : ETAP_GetState(); break;
		case CTAP_ExitNormal : ETAP_ExitNormal(); break;
		case CTAP_EnterNormal : ETAP_EnterNormal(); break;
		case CTAP_Exit : ETAP_Exit(); break;

		case CTAP_KbHit : ETAP_KbHit(); break;
		case CTAP_GetCh : ETAP_GetCh(); break;
		case CTAP_PutCh : ETAP_PutCh(); break;
		case CTAP_Print : ETAP_Print(); break;

		// -- COMMON FUNCTION ---------------------------------------------------
		case CTAP_SetSoundLevel : ETAP_SetSoundLevel(); break;
		case CTAP_GetTime : ETAP_GetTime(); break;
		case CTAP_MemInfo : ETAP_MemInfo(); break;
		case CTAP_MemAlloc : ETAP_MemAlloc(); break;
		case CTAP_MemFree : ETAP_MemFree(); break;
		case CTAP_GetTick : ETAP_GetTick(); break;
		case CTAP_Delay : ETAP_Delay(); break;
		case CTAP_ExtractMjd : ETAP_ExtractMjd(); break;
		case CTAP_MakeMjd : ETAP_MakeMjd(); break;
		case CTAP_Sin : ETAP_Sin(); break;
		case CTAP_Cos : ETAP_Cos(); break;
		case CTAP_SPrint : ETAP_SPrint(); break;
		case CTAP_GetSignalLevel : ETAP_GetSignalLevel(); break;
		case CTAP_GetSignalQuality : ETAP_GetSignalQuality(); break;
		case CTAP_SetInfoboxTime : ETAP_SetInfoboxTime(); break;
		case CTAP_PlayPCM : ETAP_PlayPCM(); break;
		case CTAP_CaptureScreen : ETAP_CaptureScreen(); break;

		case CTAP_SetSystemVar : ETAP_SetSystemVar(); break;
		case CTAP_GetSystemVar : ETAP_GetSystemVar(); break;
		case CTAP_WriteSystemVar : ETAP_WriteSystemVar(); break;
		case CTAP_GenerateEvent : ETAP_GenerateEvent(); break;

		// -- OSD FUNCTION ---------------------------------------------------
		case CTAP_Osd_Create : ETAP_Osd_Create(); break;
		case CTAP_Osd_Delete : ETAP_Osd_Delete(); break;
		case CTAP_Osd_Move : ETAP_Osd_Move(); break;
		case CTAP_Osd_FillBox : ETAP_Osd_FillBox(); break;
		case CTAP_Osd_PutBox : ETAP_Osd_PutBox(); break;
		case CTAP_Osd_GetBox : ETAP_Osd_GetBox(); break;
		case CTAP_Osd_SetLut : ETAP_Osd_SetLut(); break;
		case CTAP_Osd_Ctrl : ETAP_Osd_Ctrl(); break;
		case CTAP_Osd_SetTransparency : ETAP_Osd_SetTransparency(); break;

		case CTAP_Osd_Draw3dBox : ETAP_Osd_Draw3dBox(); break;
		case CTAP_Osd_Draw3dBoxFill : ETAP_Osd_Draw3dBoxFill(); break;

		case CTAP_Osd_DrawRectangle : ETAP_Osd_DrawRectangle(); break;
		case CTAP_Osd_DrawPixmap : ETAP_Osd_DrawPixmap(); break;
		case CTAP_Osd_Zoom : ETAP_Osd_Zoom(); break;
		case CTAP_Osd_GetBaseInfo : ETAP_Osd_GetBaseInfo(); break;
		case CTAP_Osd_Copy : ETAP_Osd_Copy(); break;
		case CTAP_Osd_PutGd : ETAP_Osd_PutGd(); break;

		case CTAP_Osd_SaveBox : ETAP_Osd_SaveBox(); break;
		case CTAP_Osd_RestoreBox : ETAP_Osd_RestoreBox(); break;

		case CTAP_Osd_GetPixel : ETAP_Osd_GetPixel(); break;
		case CTAP_Osd_PutPixel : ETAP_Osd_PutPixel(); break;

		// -- HDD FUNCTION ---------------------------------------------------

		case CTAP_Hdd_Fopen : ETAP_Hdd_Fopen(); break;
		case CTAP_Hdd_Fread : ETAP_Hdd_Fread(); break;
		case CTAP_Hdd_Fwrite : ETAP_Hdd_Fwrite(); break;
		case CTAP_Hdd_Fclose : ETAP_Hdd_Fclose(); break;
		case CTAP_Hdd_FindFirst : ETAP_Hdd_FindFirst(); break;
		case CTAP_Hdd_FindNext : ETAP_Hdd_FindNext(); break;
		case CTAP_Hdd_Fseek : ETAP_Hdd_Fseek(); break;
		case CTAP_Hdd_Flen : ETAP_Hdd_Flen(); break;
		case CTAP_Hdd_Ftell : ETAP_Hdd_Ftell(); break;
		case CTAP_Hdd_TotalSize : ETAP_Hdd_TotalSize(); break;
		case CTAP_Hdd_FreeSize : ETAP_Hdd_FreeSize(); break;
		case CTAP_Hdd_Delete : ETAP_Hdd_Delete(); break;
		case CTAP_Hdd_Exist : ETAP_Hdd_Exist(); break;
		case CTAP_Hdd_Create : ETAP_Hdd_Create(); break;
		case CTAP_Hdd_ChangeDir : ETAP_Hdd_ChangeDir(); break;

		case CTAP_Hdd_PlayTs : ETAP_Hdd_PlayTs(); break;
		case CTAP_Hdd_StopTs : ETAP_Hdd_StopTs(); break;
		case CTAP_Hdd_PlayMp3 : ETAP_Hdd_PlayMp3(); break;
		case CTAP_Hdd_StopMp3 : ETAP_Hdd_StopMp3(); break;
		case CTAP_Hdd_Rename : ETAP_Hdd_Rename(); break;


		// -- STRING FUNCTION ---------------------
		case CTAP_Osd_PutS : ETAP_Osd_PutS(); break;
		case CTAP_Osd_GetW : ETAP_Osd_GetW(); break;
		case CTAP_Osd_PutString : ETAP_Osd_PutString(); break;
		case CTAP_Osd_DrawString : ETAP_Osd_DrawString(); break;
		case CTAP_Osd_PutStringAf : ETAP_Osd_PutStringAf(); break;

		// -- WINDOW FUNCTION ---------------------------------------------------
		case CTAP_Win_SetTitle : ETAP_Win_SetTitle(); break;
		case CTAP_Win_SetColor : ETAP_Win_SetColor(); break;
		case CTAP_Win_SetDefaultColor : ETAP_Win_SetDefaultColor(); break;
		case CTAP_Win_Draw : ETAP_Win_Draw(); break;
		case CTAP_Win_Create : ETAP_Win_Create(); break;
		case CTAP_Win_Delete : ETAP_Win_Delete(); break;
		case CTAP_Win_SetFont : ETAP_Win_SetFont(); break;
		case CTAP_Win_AddItem : ETAP_Win_AddItem(); break;
		case CTAP_Win_GetSelection : ETAP_Win_GetSelection(); break;
		case CTAP_Win_SetSelection : ETAP_Win_SetSelection(); break;
		case CTAP_Win_Action : ETAP_Win_Action(); break;

		// ---------------------------------------------------------------------

		case CTAP_Channel_GetTotalNum : ETAP_Channel_GetTotalNum(); break;
		case CTAP_Channel_GetFirstInfo : ETAP_Channel_GetFirstInfo(); break;
		case CTAP_Channel_GetNextInfo : ETAP_Channel_GetNextInfo(); break;
		case CTAP_Channel_GetInfo : ETAP_Channel_GetInfo(); break;
		case CTAP_Channel_GetCurrent : ETAP_Channel_GetCurrent(); break;
		case CTAP_Channel_Start : ETAP_Channel_Start(); break;
		case CTAP_Channel_Scale : ETAP_Channel_Scale(); break;
		case CTAP_Channel_IsPlayable : ETAP_Channel_IsPlayable(); break;
		case CTAP_Channel_Move : ETAP_Channel_Move(); break;
		case CTAP_Channel_Stop : ETAP_Channel_Stop(); break;
		case CTAP_Channel_GetTotalAudioTrack : ETAP_Channel_GetTotalAudioTrack(); break;
		case CTAP_Channel_GetAudioTrackName : ETAP_Channel_GetAudioTrackName(); break;
		case CTAP_Channel_SetAudioTrack : ETAP_Channel_SetAudioTrack(); break;
		case CTAP_Channel_GetTotalSubtitleTrack : ETAP_Channel_GetTotalSubtitleTrack(); break;
		case CTAP_Channel_GetSubtitleTrackName : ETAP_Channel_GetSubtitleTrackName(); break;
		case CTAP_Channel_SetSubtitleTrack : ETAP_Channel_SetSubtitleTrack(); break;

		case CTAP_GetEvent : ETAP_GetEvent(); break;
		case CTAP_GetCurrentEvent : ETAP_GetCurrentEvent(); break;

		// --- USB FUNCTIONS ---------------------------------------------------

		case CTAP_Usb_Read : ETAP_Usb_Read(); break;
		case CTAP_Usb_Write : ETAP_Usb_Write(); break;
		case CTAP_Usb_PacketRead : ETAP_Usb_PacketRead(); break;
		case CTAP_Usb_PacketWrite : ETAP_Usb_PacketWrite(); break;
		case CTAP_Usb_Cancel : ETAP_Usb_Cancel(); break;

		// --- TIMER FUNCTIONS ---------------------------------------------------

		case CTAP_Timer_GetTotalNum : ETAP_Timer_GetTotalNum(); break;
		case CTAP_Timer_GetInfo : ETAP_Timer_GetInfo(); break;
		case CTAP_Timer_Add : ETAP_Timer_Add(); break;
		case CTAP_Timer_Modify : ETAP_Timer_Modify(); break;
		case CTAP_Timer_Delete : ETAP_Timer_Delete(); break;

		// ---------------------------------------------------------------------
		case 0xFFFF	: _exitFlag = TRUE; break;
		default:
			// here send error code
			break;
	}
}

